@@ -2,60 +2,66 @@
 
 #include "logging_categories.h"
 
-#include <Quotient/util.h>
+#include <Quotient/room.h>
 
 #include <QtGui/QTextDocument>
 #include <QtGui/QFontDatabase>
 
-#include <QtCore/QRegularExpression>
 #include <QtCore/QXmlStreamReader>
 #include <QtCore/QXmlStreamWriter>
-#include <QtCore/QStringBuilder>
-#include <QtCore/QDebug>
 
+#include <ranges>
 #include <stack>
 
 using namespace std;
+using namespace Qt::StringLiterals;
 
-namespace HtmlFilter {
+namespace {
+using namespace HtmlFilter;
+
+inline QRegularExpression operator""_qre(const char* latin1s, size_t size)
+{
+    return QRegularExpression(operator""_L1(latin1s, size));
+}
 
 enum Mode : unsigned char { QtToMatrix, MatrixToQt, GenericToQt };
 
-class Processor {
+class Processor : public QXmlStreamEntityResolver {
 public:
-    [[nodiscard]] static Result process(QString html, Mode mode,
-                                        QuaternionRoom* context,
+    [[nodiscard]] static Result process(QString html, Mode mode, const Context& context,
                                         Options options = Default);
 
 private:
     const Mode mode;
     const Options options;
-    QuaternionRoom* const context;
+    const Context& context;
     QXmlStreamWriter& writer;
-    int errorPos = -1;
+    qsizetype errorPos = -1;
     QString errorString {};
 
-    Processor(Mode mode, Options options, QuaternionRoom* context,
-              QXmlStreamWriter& writer)
+    Processor(Mode mode, Options options, const Context& context, QXmlStreamWriter& writer)
         : mode(mode), options(options), context(context), writer(writer)
     {}
+    Q_DISABLE_COPY_MOVE(Processor)
     void runOn(const QString& html);
 
     using rewrite_t = vector<pair<QString, QXmlStreamAttributes>>;
 
-    [[nodiscard]] rewrite_t filterTag(QStringView tag,
-                                      QXmlStreamAttributes attributes);
+    [[nodiscard]] rewrite_t filterTag(QStringView tag, QXmlStreamAttributes attributes);
     void filterText(QString& text);
+
+    QString resolveUndeclaredEntity(const QString& name) override
+    {
+        return name == u"nbsp" ? u"\xa0"_s : QString();
+    }
 };
 
-static const QStringView permittedTags[] = {
-    u"font",   u"del",        u"h1", u"h2", u"h3",      u"h4",     u"h5",
-    u"h6",     u"blockquote", u"p",  u"a",  u"ul",      u"ol",     u"sup",
-    u"sub",    u"li",         u"b",  u"i",  u"u",       u"strong", u"em",
-    u"strike", u"code",       u"hr", u"br", u"div",     u"table",  u"thead",
-    u"tbody",  u"tr",         u"th", u"td", u"caption", u"pre",    u"span",
-    u"img",    u"mx-reply"
-};
+constexpr auto permittedTags = std::to_array<QStringView>(
+  {u"font",       u"del", u"h1",    u"h2",     u"h3",      u"h4",  u"h5",   u"h6",
+   u"blockquote", u"p",   u"a",     u"ul",     u"ol",      u"sup", u"sub",  u"li",
+   u"b",          u"i",   u"u",     u"strong", u"em",      u"s",   u"code", u"hr",
+   u"br",         u"div", u"table", u"thead",  u"tbody",   u"tr",  u"th",   u"td",
+   u"caption",    u"pre", u"span",  u"img",    u"mx-reply"});
 
 struct PassList {
     QStringView tag;
@@ -63,25 +69,32 @@ struct PassList {
 };
 
 // See filterTag() on special processing of commented out tags/attributes
-static const PassList passLists[] = {
-    { u"a", { u"name", u"target", /* "href" - only from permittedSchemes */ } },
-    { u"img", { u"width", u"height", u"alt", u"title", u"data-mx-emoticon"
-               /* "src" - only 'mxc:' */ } },
-    { u"ol", { u"start" } },
-    { u"font", { u"color", u"data-mx-color", u"data-mx-bg-color" } },
-    { u"span", { u"color", u"data-mx-color", u"data-mx-bg-color" } }
-    //, { "code", { "class" /* must start with 'language-' */ } } // Special case
-};
+const auto passLists = std::to_array<PassList>({
+  {u"a", {u"name", u"target", /* u"href" - only from permittedSchemes */}},
+  {u"img", {u"width", u"height", u"alt", u"title", u"data-mx-emoticon", /* u"src" - only 'mxc:' */}},
+  {u"ol", {u"start"}},
+  {u"font", {u"color", u"data-mx-color", u"data-mx-bg-color"}},
+  {u"span", {u"color", u"data-mx-color", u"data-mx-bg-color"}},
+  // { u"code", { u"class" /* must start with 'language-' */ } }
+});
 
-static QStringView const permittedSchemes[]{
-    u"http:",   u"https:",  u"ftp:", u"mailto:",
-    u"magnet:", u"matrix:", u"mxc:" /* MSC2398 */
-};
+constexpr auto permittedSchemes = std::to_array<QStringView>({
+  u"http:", u"https:", u"ftp:", u"mailto:", u"magnet:", u"matrix:", u"mxc:" /* MSC2398 */
+});
 
-static const auto htmlColorAttr = u"color";
-static const auto htmlStyleAttr = u"style";
-static const auto mxColorAttr = u"data-mx-color";
-static const auto mxBgColorAttr = u"data-mx-bg-color";
+constexpr auto htmlColorAttr = u"color";
+constexpr auto htmlStyleAttr = u"style";
+constexpr auto mxColorAttr = u"data-mx-color";
+constexpr auto mxBgColorAttr = u"data-mx-bg-color";
+
+#ifdef __cpp_lib_ranges_contains
+constexpr auto rangeContains = ranges::contains;
+#else
+inline auto rangeContains(const auto& c, const auto& v)
+{
+    return std::ranges::find(c, v) != std::ranges::end(c);
+}
+#endif
 
 [[nodiscard]] QString mergeMarkdown(const QString& html)
 {
@@ -95,12 +108,10 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
     QXmlStreamReader reader(html);
     QString mdWithHtml;
     QXmlStreamWriter writer(&mdWithHtml);
-    while (reader.readNext() != QXmlStreamReader::StartElement
-               || reader.qualifiedName() != u"p")
+    while (reader.readNext() != QXmlStreamReader::StartElement || reader.qualifiedName() != u"p")
         if (reader.atEnd()) {
             Q_ASSERT_X(false, __FUNCTION__, "Malformed Qt markup");
-            qCCritical(HTMLFILTER)
-                << "The passed text doesn't seem to come from QTextDocument";
+            qCCritical(HTMLFILTER) << "The passed text doesn't seem to come from QTextDocument";
             return {};
         }
 
@@ -141,8 +152,7 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
             qCWarning(HTMLFILTER) << "Unexpected token, type" << tokenType;
         }
         if (depth < 0) {
-            Q_ASSERT(tokenType == QXmlStreamReader::EndElement
-                     && reader.qualifiedName() == u"body");
+            Q_ASSERT(tokenType == QXmlStreamReader::EndElement && reader.qualifiedName() == u"body");
             break;
         }
         writer.writeCurrentToken(reader);
@@ -176,12 +186,11 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
     bool inHead = false;
     for (auto pos = html.indexOf('<'); pos != -1; pos = html.indexOf('<', pos)) {
         const auto tagNamePos = pos + 1 + (html[pos + 1] == '/');
-        const QStringView uncheckedHtml = QStringView(html).mid(tagNamePos);
-        static const QLatin1String commentOpen("!--");
-        static const QLatin1String commentClose("-->");
+        const auto uncheckedHtml = QStringView(html).mid(tagNamePos);
+        static constexpr auto commentOpen = "!--"_L1;
+        static constexpr auto commentClose = "-->"_L1;
         if (uncheckedHtml.startsWith(commentOpen)) { // Skip comments
-            pos = html.indexOf(commentClose, tagNamePos + commentOpen.size())
-                  + commentClose.size();
+            pos = html.indexOf(commentClose, tagNamePos + commentOpen.size()) + commentClose.size();
             continue;
         }
         // Look ahead to detect stray < and escape it
@@ -190,7 +199,7 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
         if (gtPos == tagNamePos /* <> or </> */ || gtPos == -1 /* no more > */
             || ((nextLtPos = html.indexOf('<', tagNamePos)) != -1
                 && nextLtPos < gtPos) /* there's another < before > */) {
-            static const auto to = QStringLiteral("&lt;");
+            static const auto to = u"&lt;"_s;
             html.replace(pos, 1, to);
             pos += to.size(); // Put pos after the escaped sequence
             continue;
@@ -201,12 +210,9 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
                 // user input (Validate is on) or remove the whole header if
                 // it comes from the wire (Validate is off).
                 if (options.testFlag(Validate))
-                    return { {},
-                             pos,
-                             "<head> elements are not allowed in Matrix" };
-                static const QLatin1String HeadEnd("</head>");
-                const auto headEndPos =
-                    html.indexOf(HeadEnd, tagNamePos, Qt::CaseInsensitive);
+                    return { {}, pos, u"<head> elements are not allowed in Matrix"_s };
+                static constexpr auto HeadEnd = "</head>"_L1;
+                const auto headEndPos = html.indexOf(HeadEnd, tagNamePos, Qt::CaseInsensitive);
                 html.remove(pos, headEndPos - pos + HeadEnd.size());
                 continue;
             }
@@ -218,12 +224,8 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
             }
         }
 
-        const auto tagEndIt = find_if(uncheckedHtml.cbegin(),
-                                      uncheckedHtml.cend(), isTagNameTerminator);
-        const auto tag =
-            uncheckedHtml.left(int(tagEndIt - uncheckedHtml.cbegin()))
-                .toString()
-                .toLower();
+        const auto tagEndIt = ranges::find_if(uncheckedHtml, isTagNameTerminator);
+        const auto tag = uncheckedHtml.left(tagEndIt - uncheckedHtml.cbegin()).toString().toLower();
         // <head> contents are necessary to apply styles but obviously
         // neither `head` nor tags inside of it are in permittedTags;
         // however, minimised attributes still have to be handled everywhere
@@ -234,51 +236,42 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
             isFragment = false;
             continue;
         }
-        if (!inHead) {
-            // Check if it's a valid (opening or closing) tag allowed in Matrix
-            const auto tagIt =
-                find(cbegin(permittedTags), cend(permittedTags), tag);
-            if (tagIt == cend(permittedTags)) {
-                // Invalid tag or non-tag - either remove the abusing piece
-                // or stop and report
-                if (options.testFlag(Validate))
-                    return { {},
-                             pos,
-                             "Non-tag or disallowed tag: "
-                                 % uncheckedHtml.left(gtPos - tagNamePos) };
+        // Check if it's a valid (opening or closing) tag allowed in Matrix
+        if (!inHead && !rangeContains(permittedTags, tag)) {
+            // Invalid tag or non-tag - either remove the abusing piece or stop and report
+            if (options.testFlag(Validate))
+                return {{},
+                        pos,
+                        u"Non-tag or disallowed tag: "_s % uncheckedHtml.left(gtPos - tagNamePos)};
 
-                html.remove(pos, gtPos - pos + 1);
-                continue;
-            }
+            html.remove(pos, gtPos - pos + 1);
+            continue;
         }
 
         // Treat minimised attributes
         // (https://www.w3.org/TR/xhtml1/diffs.html#h-4.5)
 
-        // There's no simple way to replace all occurences within
+        // There's no simple way to replace all occurrences within
         // a string segment; so just go through the segment and insert
         // `=''` after minimized attributes.
         // This is not the place to _filter_ allowed/disallowed attributes -
         // filtering is left for filterTag()
-        static const QRegularExpression MinAttrRE {
-            R"(([^[:space:]>/"'=]+)\s*(=\s*([^[:space:]>/"']|"[^"]*"|'[^']*')+)?)"
-        };
+        static const auto MinAttrRE =
+          R"(([^[:space:]>/"'=]+)\s*(=\s*([^[:space:]>/"']|"[^"]*"|'[^']*')+)?)"_qre;
         pos = tagNamePos + tag.size();
         QRegularExpressionMatch m;
-        while ((m = MinAttrRE.match(html, pos)).hasMatch()
-               && m.capturedEnd(1) < gtPos) {
+        while ((m = MinAttrRE.match(html, pos)).hasMatch() && m.capturedEnd(1) < gtPos) {
             pos = m.capturedEnd();
             if (m.captured(2).isEmpty()) {
-                static const auto attrValue = QString("=''");
+                static const auto attrValue = u"=''"_s;
                 html.insert(m.capturedEnd(1), attrValue);
-                gtPos += attrValue.size() - 1;
-                pos += attrValue.size() - 1;
+                gtPos += attrValue.size();
+                pos += attrValue.size();
             }
         }
         // Make sure empty elements are properly closed
-        static const QRegularExpression EmptyElementRE {
-            "^img|[hb]r|meta$", QRegularExpression::CaseInsensitiveOption
-        };
+        static const QRegularExpression EmptyElementRE{"^img|[hb]r|meta$"_L1,
+                                                       QRegularExpression::CaseInsensitiveOption};
         if (html[gtPos - 1] != '/' && EmptyElementRE.match(tag).hasMatch()) {
             html.insert(gtPos, '/');
             ++gtPos;
@@ -296,8 +289,7 @@ static const auto mxBgColorAttr = u"data-mx-bg-color";
     return { html };
 }
 
-Result Processor::process(QString html, Mode mode, QuaternionRoom* context,
-                          Options options)
+Result Processor::process(QString html, Mode mode, const Context& context, Options options)
 {
     // Since Qt doesn't have an HTML parser (outside of QTextDocument; and
     // the one in QTextDocument is opinionated and not configurable)
@@ -315,10 +307,27 @@ Result Processor::process(QString html, Mode mode, QuaternionRoom* context,
     // characters.
 
     // 1. Escape ampersands outside of character entities
-    static const QRegularExpression freestandingAmps{ QStringLiteral(
-        "&(?!(#[0-9]+|#x[0-9a-fA-F]+|[[:alpha:]_][-[:alnum:]_:.]*);)") };
+    static const auto freestandingAmps =
+      "&(?!(#[0-9]+|#x[0-9a-fA-F]+|[[:alpha:]_][-[:alnum:]_:.]*);)"_qre;
     html.replace(freestandingAmps, QStringLiteral("&amp;"));
 
+    if (mode != GenericToQt) {
+        // Handling control codes (excluding, for this discussion, \n, \r, and \t) in HTML is
+        // somewhat messy. HTML 4 and XML 1.0 and XHTML 1.0 all disallow C0/C1 control codes in any
+        // form. XML 1.1 allows them as numeric character references (aka NCRs) but
+        // QXmlStreamReader only implements XML 1.0 and doesn't accept them even as NCRs.
+        // Meanwhile, QTextDocument emits control codes to HTML without any conversion, formally
+        // violating HTML 4 spec (https://bugreports.qt.io/browse/QTBUG-122466) and, more
+        // importantly for this code, upsetting QXmlStreamReader (#900). HTML 5 (which Matrix HTML
+        // is - assumed to be - based on) formally disallows control codes too, adding \f to the
+        // allowed exclusions (see https://dev.w3.org/html5/spec-LC/syntax.html#text-0) which gives
+        // us the right to eliminate control characters from Matrix payloads, even though the Web
+        // generally seems to admit them as NCRs.
+        // NB: [:cntrl:] doesn't work because it includes the allowed \n, \r, \t
+        static const auto controlCharRE = R"([\x01-\x08\x0b\x0c\x0e-\x1f\x7f-\x9f])"_qre;
+        html.remove(controlCharRE);
+    }
+
     if (mode == QtToMatrix) {
         if (options.testFlag(ConvertMarkdown)) {
             // The processor handles Markdown in chunks between HTML tags;
@@ -347,56 +356,10 @@ Result Processor::process(QString html, Mode mode, QuaternionRoom* context,
     return { resultHtml.trimmed(), p.errorPos, p.errorString };
 }
 
-QString toMatrixHtml(const QString& qtMarkup, QuaternionRoom* context,
-                     Options options)
-{
-    // Validation of HTML emitted by Qt doesn't make much sense
-    Q_ASSERT(!options.testFlag(Validate));
-    const auto& result =
-        Processor::process(qtMarkup, QtToMatrix, context, options);
-    Q_ASSERT(result.errorPos == -1);
-    return result.filteredHtml;
-}
-
-Result fromMatrixHtml(const QString& matrixHtml, QuaternionRoom* context,
-                      Options options)
-{
-    // Matrix HTML body should never be treated as Markdown
-    Q_ASSERT(!options.testFlag(ConvertMarkdown));
-    auto result = Processor::process(matrixHtml, MatrixToQt, context, options);
-    if (result.errorPos == -1) {
-        // Make sure to preserve whitespace sequences
-        result.filteredHtml = "<span style=\"white-space: pre-wrap\">"
-                              % result.filteredHtml % "</span>";
-    }
-    return result;
-}
-
-Result fromLocalHtml(const QString& html,
-                     QuaternionRoom* context, Options options)
-{
-    return Processor::process(html, GenericToQt, context, options);
-}
-
-namespace {
-    class EntityResolver : public QXmlStreamEntityResolver {
-    public:
-        using QXmlStreamEntityResolver::QXmlStreamEntityResolver;
-        Q_DISABLE_COPY_MOVE(EntityResolver)
-
-    private:
-        QString resolveUndeclaredEntity(const QString& name) override
-        {
-            return name == u"nbsp" ? QStringLiteral("\xa0") : QString();
-        }
-    };
-    EntityResolver entityResolver;
-}
-
 void Processor::runOn(const QString &html)
 {
     QXmlStreamReader reader(html);
-    reader.setEntityResolver(&entityResolver);
+    reader.setEntityResolver(this);
 
     /// The entry in the (outer) stack corresponds to each level in the source
     /// document; the (inner) stack in each entry records open elements in the
@@ -408,15 +371,14 @@ void Processor::runOn(const QString &html)
     /// tag (opening or closing); used to linkify (or process Markdown in)
     /// text parts.
     QString textBuffer;
-    int bodyOffset = 0;
+    decltype(reader.characterOffset()) bodyOffset = 0;
     bool firstElement = true, inAnchor = false;
     while (!reader.atEnd()) {
         const auto tokenType = reader.readNext();
         if (bodyOffset == -1) // See below in 'case StartElement:'
             bodyOffset = reader.characterOffset();
 
-        if (!textBuffer.isEmpty() && !reader.isCharacters()
-            && !reader.isEntityReference())
+        if (!textBuffer.isEmpty() && !reader.isCharacters() && !reader.isEntityReference())
             filterText(textBuffer);
 
         switch (tokenType) {
@@ -462,32 +424,31 @@ void Processor::runOn(const QString &html)
                     break;
                 }
             }
+            if (options.testFlag(StripMxReply) && tagName == u"mx-reply") {
+                reader.skipCurrentElement();
+                continue;
+            }
 
             const auto& attrs = reader.attributes();
-            if (find_if(attrs.cbegin(), attrs.cend(),
-                        [](const auto& a) {
-                            return a.qualifiedName() == u"style"
-                                   && a.value().contains(
-                                       u"-qt-paragraph-type:empty");
-                        })
-                != attrs.cend()) {
+            if (ranges::any_of(attrs, [](const auto& a) {
+                    return a.qualifiedName() == u"style"
+                           && a.value().contains(u"-qt-paragraph-type:empty");
+                })) { // Hidden text block, just skip it
                 reader.skipCurrentElement();
-                continue; // Hidden text block, just skip it
+                continue;
             }
 
             tagsStack.emplace();
             if (tagsStack.size() > 100)
-                qCCritical(HTMLFILTER)
-                    << "CS API spec limits HTML tags depth at 100";
+                qCCritical(HTMLFILTER) << "CS API spec limits HTML tags depth at 100";
 
             // Qt hardcodes the link style in a `<span>` under `<a>`.
             // This breaks the looks on the receiving side if the sender
             // uses a different style of links from that of the receiver.
             // Since Qt decorates links when importing HTML anyway, we
             // don't lose anything if we just strip away this span tag.
-            if (mode != MatrixToQt && inAnchor && textBuffer.isEmpty()
-                && tagName == u"span" && attrs.size() == 1
-                && attrs.front().qualifiedName() == u"style")
+            if (mode != MatrixToQt && inAnchor && textBuffer.isEmpty() && tagName == u"span"
+                && attrs.size() == 1 && attrs.front().qualifiedName() == u"style")
                 continue; // inAnchor == true ==> firstElement == false
 
             // Skip the first top-level <p> and replace further top-level
@@ -503,11 +464,10 @@ void Processor::runOn(const QString &html)
                 && tagsStack.size() == 1 /* top-level, just emplaced */) {
                 if (firstElement)
                     continue; // Skip unsetting firstElement at the loop end
-                writer.writeEmptyElement("br");
+                writer.writeEmptyElement(u"br"_s);
                 break;
             }
-            if (tagName != u"mx-reply"
-                || (firstElement && !options.testFlag(Fragment))) {
+            if (tagName != u"mx-reply" || (firstElement && !options.testFlag(Fragment))) {
                 // ^ The spec only allows `<mx-reply>` at the very beginning
                 // and it's not supposed to be in the user input
                 const auto& rewrite = filterTag(tagName, attrs);
@@ -515,7 +475,7 @@ void Processor::runOn(const QString &html)
                     tagsStack.top().push(rewrittenTag);
                     writer.writeStartElement(rewrittenTag);
                     writer.writeAttributes(rewrittenAttrs);
-                    if (rewrittenTag == "a")
+                    if (rewrittenTag == u"a")
                         inAnchor = true;
                 }
             }
@@ -546,22 +506,20 @@ void Processor::runOn(const QString &html)
                 const auto& tagName = reader.qualifiedName();
                 if (tagName != u"body" && tagName != u"html")
                     qCWarning(HTMLFILTER)
-                        << "filterHtml(): empty tags stack, skipping"
-                        << ('/' + tagName.toString());
+                      << "Empty tags stack, skipping" << ('/' + tagName.toString());
                 break;
             }
             // Close as many elements as were opened in case StartElement
             for (auto& t = tagsStack.top(); !t.empty(); t.pop()) {
                 writer.writeEndElement();
-                if (t.top() == "a")
+                if (t.top() == u"a")
                     inAnchor = false;
             }
             tagsStack.pop();
             break;
         case QXmlStreamReader::EndDocument:
             if (!tagsStack.empty())
-                qCWarning(HTMLFILTER).noquote().nospace()
-                    << ": Not all HTML tags closed";
+                qCWarning(HTMLFILTER) << "Not all HTML tags closed at the document end";
             if (mode == GenericToQt)
                 writer.writeEndDocument(); // </body></html>
             break;
@@ -572,14 +530,11 @@ void Processor::runOn(const QString &html)
             errorPos = reader.characterOffset() - bodyOffset;
             errorString = reader.errorString();
             qCCritical(HTMLFILTER) << "Invalid XHTML:" << html;
+            qCCritical(HTMLFILTER).nospace() << "Error at char " << errorPos << ": " << errorString;
+            const auto remainder = QStringView(html).mid(reader.characterOffset());
             qCCritical(HTMLFILTER).nospace()
-                << "Error at char " << errorPos << ": " << errorString;
-            const auto remainder =
-                QStringView(html).mid(reader.characterOffset());
-            qCCritical(HTMLFILTER).nospace()
-                << "Buffer at error: " << remainder << ", "
-                << html.size() - reader.characterOffset()
-                << " character(s) remaining";
+              << "Buffer at error: " << remainder << ", " << html.size() - reader.characterOffset()
+              << " character(s) remaining";
             break;
         }
         case QXmlStreamReader::Comment:
@@ -595,61 +550,49 @@ void Processor::runOn(const QString &html)
 }
 
 template <size_t Len>
-inline QStringView cssValue(QStringView css,
-                            const char16_t (&propertyNameWithColon)[Len])
+inline QStringView cssValue(QStringView css, const char16_t (&propertyNameWithColon)[Len])
 {
-    return css.startsWith(propertyNameWithColon)
-               ? css.mid(Len - 1).trimmed()
-               : QStringView();
+    return css.startsWith(propertyNameWithColon) ? css.mid(Len - 1).trimmed() : QStringView();
 }
 
-Processor::rewrite_t Processor::filterTag(QStringView tag,
-                                          QXmlStreamAttributes attributes)
+Processor::rewrite_t Processor::filterTag(QStringView tag, QXmlStreamAttributes attributes)
 {
     if (mode == MatrixToQt) {
         if (tag == u"del" || tag == u"strike") { // Qt doesn't support these...
             QXmlStreamAttributes attrs;
-            attrs.append("style", "text-decoration:line-through");
-            return { { "font", std::move(attrs) } };
+            attrs.append(u"style"_s, u"text-decoration:line-through"_s);
+            return { { u"font"_s, std::move(attrs) } };
         }
         if (tag == u"mx-reply")
-            return { { "div", {} } }; // The spec says that mx-reply is HTML div
+            return { { u"div"_s, {} } }; // The spec says that mx-reply is HTML div
         // If `mx-reply` is encountered on the way to the wire, just pass it
     }
 
     rewrite_t rewrite { { tag.toString(), {} } };
     if (tag == u"code" && mode != GenericToQt) { // Special case
-        copy_if(attributes.begin(), attributes.end(),
-                back_inserter(rewrite.back().second), [](const auto& a) {
-                    return a.qualifiedName() == u"class"
-                           && a.value().startsWith(u"language-");
-                });
+        ranges::copy_if(attributes, back_inserter(rewrite.back().second), [](const auto& a) {
+            return a.qualifiedName() == u"class" && a.value().startsWith(u"language-");
+        });
         return rewrite;
     }
 
-    if (find(begin(permittedTags), end(permittedTags), tag) == end(permittedTags))
+    if (!rangeContains(permittedTags, tag))
         return {}; // The tag is not allowed
 
-    const auto it =
-        find_if(begin(passLists), end(passLists),
-                [&tag](const auto& passCard) { return passCard.tag == tag; });
+    const auto it = ranges::find(passLists, tag, &PassList::tag);
     if (it == end(passLists))
         return rewrite; // Drop all attributes, pass the tag
 
     /// Find the first element in the rewrite that would accept color
     /// attributes (`font` and, only in Matrix HTML, `span`),
     /// and add the passed attribute to it
-    const auto& addColorAttr = [&rewrite, this](QStringView attrName,
-                                                QStringView attrValue) {
-        auto it = find_if(rewrite.begin(), rewrite.end(),
-                          [this](const rewrite_t::value_type& element) {
-                              return element.first == "font"
-                                     || (mode == QtToMatrix
-                                         && element.first == "span");
-                          });
-        if (it == rewrite.end())
-            it = rewrite.insert(rewrite.end(), { QStringLiteral("font"), {} });
-        it->second.append(attrName.toString(), attrValue.toString());
+    const auto& addColorAttr = [&rewrite, this](QStringView attrName, QStringView attrValue) {
+        auto colourableIt = ranges::find_if(rewrite, [this](const rewrite_t::value_type& element) {
+            return element.first == "font" || (mode == QtToMatrix && element.first == "span");
+        });
+        if (colourableIt == rewrite.end())
+            colourableIt = rewrite.insert(rewrite.end(), { u"font"_s, {} });
+        colourableIt->second.append(attrName.toString(), attrValue.toString());
     };
 
     const auto& passList = it->allowedAttrs;
@@ -665,8 +608,7 @@ Processor::rewrite_t Processor::filterTag(QStringView tag,
             }
             if (aName == mxBgColorAttr) {
                 rewrite.front().second.append(QString::fromUtf16(htmlStyleAttr),
-                                              "background-color:"
-                                                  + aValue.toString());
+                                              "background-color:" + aValue.toString());
                 continue;
             }
         } else {
@@ -680,29 +622,25 @@ Processor::rewrite_t Processor::filterTag(QStringView tag,
                         continue;
                     if (const auto& v = cssValue(p, u"color:"); !v.isEmpty()) {
                         addColorAttr(mxColorAttr, v);
-                    } else if (const auto& v = cssValue(p, u"background-color:");
-                               !v.isEmpty())
+                    } else if (const auto& v = cssValue(p, u"background-color:"); !v.isEmpty())
                         addColorAttr(mxBgColorAttr, v);
                     else if (const auto& v = cssValue(p, u"font-weight:");
                              v == u"bold" || v == u"bolder" || v.toFloat() > 500)
-                        rewrite.emplace_back().first = "b";
+                        rewrite.emplace_back().first = u"b"_s;
                     else if (const auto& v = cssValue(p, u"font-style:");
                              v == u"italic" || v.startsWith(u"oblique"))
-                        rewrite.emplace_back().first = "i";
+                        rewrite.emplace_back().first = u"i"_s;
                     else if (const auto& v = cssValue(p, u"text-decoration:");
                              v.contains(u"line-through"))
-                        rewrite.emplace_back().first = QStringLiteral("del");
+                        rewrite.emplace_back().first = u"del"_s;
                     else {
-                        const auto& fontFamilies =
-                            cssValue(p, u"font-family:").split(',');
-                        for (auto ff: fontFamilies) {
-                            ff = ff.trimmed();
-                            if (ff.isEmpty())
-                                continue;
-                            if (ff[0] == '\'' || ff[0] == '"')
-                                ff = ff.mid(1, ff.length() - 2);
-                            if (QFontDatabase().isFixedPitch(ff.toString())) {
-                                rewrite.emplace_back().first = "code";
+                        const auto& fontFamilies = cssValue(p, u"font-family:").split(',');
+                        for (auto ff : views::transform(fontFamilies, &QStringView::trimmed)
+                                         | views::filter(std::not_fn(&QStringView::empty))) {
+                            if (ff.front() == '\'' || ff.front() == '"')
+                                ff = ff.mid(1, ff.size() - 2);
+                            if (QFontDatabase::isFixedPitch(ff.toString())) {
+                                rewrite.emplace_back().first = u"code"_s;
                                 break;
                             }
                         }
@@ -714,17 +652,32 @@ Processor::rewrite_t Processor::filterTag(QStringView tag,
                 addColorAttr(mxColorAttr, aValue); // Add to 'color'
         }
 
+        // Enrich mxc source URLs for images with the context so that NAM could resolve them
+        if (tag == u"img" && aName == u"src" && aValue.startsWith(u"mxc:")) {
+            auto url = QUrl::fromUserInput(aValue.toString());
+            if (mode == QtToMatrix) {
+                // Make sure the mxc URL is just that, with no internal extras
+                QUrlQuery q{url.query()};
+                for (const auto& k : {u"user_id"_s, u"room_id"_s, u"event_id"_s})
+                    q.removeAllQueryItems(k);
+                url.setQuery(q);
+                a = QXmlStreamAttribute(aName.toString(), url.toString(QUrl::FullyEncoded));
+            } else if (context.room) {
+                a = QXmlStreamAttribute(aName.toString(),
+                                        context.room
+                                          ->makeMediaUrl(context.eventId,
+                                                         QUrl::fromUserInput(aValue.toString()))
+                                          .toString(QUrl::FullyEncoded));
+            }
+            rewrite.front().second.push_back(std::move(a));
+        }
+
         // Generic filtering for attributes
-        if ((mode == GenericToQt
-             && (aName == htmlStyleAttr || aName == u"class" || aName == u"id"))
+        if ((mode == GenericToQt && (aName == htmlStyleAttr || aName == u"class" || aName == u"id"))
             || (tag == u"a" && aName == u"href"
-                && any_of(begin(permittedSchemes), end(permittedSchemes),
-                          [&aValue](QStringView s) {
-                              return aValue.startsWith(s);
-                          }))
-            || (tag == u"img" && aName == u"src" && aValue.startsWith(u"mxc:"))
-            || find(passList.begin(), passList.end(), a.qualifiedName())
-                   != passList.end())
+                && ranges::any_of(permittedSchemes,
+                                  [&aValue](QStringView s) { return aValue.startsWith(s); }))
+            || rangeContains(passList, a.qualifiedName()))
             rewrite.front().second.push_back(std::move(a));
     } // for (a: attributes)
 
@@ -748,7 +701,7 @@ void Processor::filterText(QString& text)
         // specific string doesn't matter as long as it isn't whitespace itself,
         // doesn't have special meaning in Markdown and doesn't occur in
         // the HTML boilerplate that QTextDocument generates.
-        static const QLatin1String Marker { "$$" };
+        static constexpr auto Marker = "$$"_L1;
         const bool hasLeadingWhitespace = text.cbegin()->isSpace();
         if (hasLeadingWhitespace)
             text.prepend(Marker);
@@ -762,9 +715,9 @@ void Processor::filterText(QString& text)
         // (see also https://spec.commonmark.org/0.29/#list-items)
         static const auto ReOptions = QRegularExpression::MultilineOption;
         static const QRegularExpression //
-            UlRE("^( *[-+*] {1,4})(?=[^ ])", ReOptions),
-            OlRE("^( *[0-9]{1,9}+[.)] {1,4})(?=[^ ])", ReOptions);
-        static const QLatin1String UlMarker("@@ul@@"), OlMarker("@@ol@@");
+          UlRE(u"^( *[-+*] {1,4})(?=[^ ])"_s, ReOptions),
+          OlRE(u"^( *[0-9]{1,9}+[.)] {1,4})(?=[^ ])"_s, ReOptions);
+        static constexpr auto UlMarker = "@@ul@@"_L1, OlMarker = "@@ol@@"_L1;
         text.replace(UlRE, "\\1" % UlMarker);
         text.replace(OlRE, "\\1" % OlMarker);
         const auto markerCountOl = text.count(OlMarker);
@@ -801,5 +754,35 @@ void Processor::filterText(QString& text)
 
     text.clear();
 }
+}
+
+namespace HtmlFilter {
+
+QString toMatrixHtml(const QString& qtMarkup, const Context& context, Options options)
+{
+    // Validation of HTML emitted by Qt doesn't make much sense
+    Q_ASSERT(!options.testFlag(Validate));
+    const auto& result = Processor::process(qtMarkup, QtToMatrix, context, options);
+    Q_ASSERT(result.errorPos == -1);
+    return result.filteredHtml;
+}
+
+Result fromMatrixHtml(const QString& matrixHtml, const Context& context, Options options)
+{
+    // Matrix HTML body should never be treated as Markdown
+    Q_ASSERT(!options.testFlag(ConvertMarkdown));
+    auto result = Processor::process(matrixHtml, MatrixToQt, context, options);
+    if (result.errorPos == -1) {
+        // Make sure to preserve whitespace sequences
+        result.filteredHtml =
+            "<span style=\"white-space: pre-wrap\">" % result.filteredHtml % "</span>";
+    }
+    return result;
+}
+
+Result fromLocalHtml(const QString& html, const Context& context, Options options)
+{
+    return Processor::process(html, GenericToQt, context, options);
+}
 
 } // namespace HtmlFilter
