@@ -1,30 +1,21 @@
 /**************************************************************************
  *                                                                        *
- * Copyright (C) 2015 Felix Rohrbach <kde@fxrh.de>                        *
+ * SPDX-FileCopyrightText: 2015 Felix Rohrbach <kde@fxrh.de>              *
  *                                                                        *
- * This program is free software; you can redistribute it and/or          *
- * modify it under the terms of the GNU General Public License            *
- * as published by the Free Software Foundation; either version 3         *
- * of the License, or (at your option) any later version.                 *
- *                                                                        *
- * This program is distributed in the hope that it will be useful,        *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- * GNU General Public License for more details.                           *
- *                                                                        *
- * You should have received a copy of the GNU General Public License      *
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *                                                                        *
  **************************************************************************/
 
 #pragma once
 
-#include "accountregistry.h"
-
-#include <uriresolver.h>
-
+// QSslError is used in a signal container parameter and needs to be complete
+// for moc to generate stuff since Qt 6
+#include <QtNetwork/QSslError>
 #include <QtWidgets/QMainWindow>
 
+#include <Quotient/accountregistry.h>
+#include <Quotient/uriresolver.h>
+
 namespace Quotient {
     class Room;
     class Connection;
@@ -47,7 +38,6 @@ class QLabel;
 class QLineEdit;
 
 class QNetworkReply;
-class QSslError;
 class QNetworkProxy;
 class QAuthenticator;
 
@@ -59,9 +49,11 @@ class MainWindow: public QMainWindow, public Quotient::UriResolverBase {
         MainWindow();
         ~MainWindow() override;
 
-        void addConnection(Connection* c, const QString& deviceName);
+        void addConnection(Connection* c);
         void dropConnection(Connection* c);
 
+        Quotient::AccountRegistry* registry() { return accountRegistry; }
+
         // For openUserInput()
         enum : bool { NoRoomJoining = false, ForJoining = true };
 
@@ -76,14 +68,15 @@ class MainWindow: public QMainWindow, public Quotient::UriResolverBase {
         void openRoomSettings(QuaternionRoom* r = nullptr);
         void selectRoom(Quotient::Room* r);
         void showStatusMessage(const QString& message, int timeout = 0);
-        void logout(Connection* c);
+        QFuture<QString> logout(Connection* c);
 
     private slots:
         void invokeLogin();
 
-        void loginError(Connection* c, const QString& message = {});
+        void reloginNeeded(Connection* c, const QString& message = {});
         void networkError(Connection* c);
-        void sslErrors(QNetworkReply* reply, const QList<QSslError>& errors);
+        void sslErrors(const QPointer<QNetworkReply>& reply,
+                       const QList<QSslError>& errors);
         void proxyAuthenticationRequired(const QNetworkProxy& /* unused */,
                                          QAuthenticator* auth);
 
@@ -102,9 +95,9 @@ class MainWindow: public QMainWindow, public Quotient::UriResolverBase {
         bool visitNonMatrix(const QUrl& url) override;
 
     private:
-        AccountRegistry accountRegistry;
+        Quotient::AccountRegistry* accountRegistry =
+            new Quotient::AccountRegistry(this);
         QVector<Connection*> logoutOnExit;
-        QVector<Connection*> firstSyncing;
 
         RoomListDock* roomListDock = nullptr;
         UserListDock* userListDock = nullptr;
@@ -132,23 +125,19 @@ class MainWindow: public QMainWindow, public Quotient::UriResolverBase {
         QAction* addUiOptionCheckbox(QMenu* parent, const QString& text,
             const QString& statusTip, const QString& settingsKey,
             bool defaultValue = false);
-        void showFirstSyncIndicator();
-        void firstSyncOver(Connection* c);
+        void showInitialLoadIndicator();
+        void updateLoadingStatus(int accountsStillLoading);
+        void firstSyncOver(const Connection *c);
         void loadSettings();
         void saveSettings() const;
         void doOpenLoginDialog(LoginDialog* dialog);
-        QByteArray loadAccessToken(const Quotient::AccountSettings& account);
-        QByteArray loadAccessTokenFromKeyChain(const Quotient::AccountSettings &account);
-        bool saveAccessToken(const Quotient::AccountSettings& account,
-                             const QByteArray& accessToken);
-        bool saveAccessTokenToFile(const Quotient::AccountSettings& account,
-                                   const QByteArray& accessToken);
-        bool saveAccessTokenToKeyChain(const Quotient::AccountSettings& account,
-                                       const QByteArray& accessToken, bool writeToFile = true);
         Connection* chooseConnection(Connection* connection,
                                      const QString& prompt);
         void showMillisToRecon(Connection* c);
 
+        std::pair<QByteArray, bool> loadAccessToken(
+            const Quotient::AccountSettings& account);
+
         /// Get the default connection to perform actions
         /*!
          * \return the connection of the current room; or, if there's only
