@@ -19,15 +19,16 @@
 #include <QtWidgets/QAbstractItemView>
 
 #include <Quotient/connection.h>
+#include <Quotient/ranges_extras.h>
 #include <Quotient/room.h>
 #include <Quotient/user.h>
 
+using Quotient::RoomMember;
+
 UserListModel::UserListModel(QAbstractItemView* parent)
     : QAbstractListModel(parent), m_currentRoom(nullptr)
 { }
 
-UserListModel::~UserListModel() = default;
-
 void UserListModel::setRoom(Quotient::Room* room)
 {
     if (m_currentRoom == room)
@@ -38,32 +39,29 @@ void UserListModel::setRoom(Quotient::Room* room)
     if (m_currentRoom) {
         m_currentRoom->connection()->disconnect(this);
         m_currentRoom->disconnect(this);
-        for (auto* user: std::as_const(m_users))
-            user->disconnect(this);
-        m_users.clear();
+        m_memberIds.clear();
     }
     m_currentRoom = room;
     if (m_currentRoom) {
-        connect(m_currentRoom, &Room::userAdded, this, &UserListModel::userAdded);
-        connect(m_currentRoom, &Room::userRemoved, this, &UserListModel::userRemoved);
-        connect(m_currentRoom, &Room::memberAboutToRename, this, &UserListModel::userRemoved);
-        connect(m_currentRoom, &Room::memberRenamed, this, &UserListModel::userAdded);
+        connect(m_currentRoom, &Room::memberJoined, this, &UserListModel::userAdded);
+        connect(m_currentRoom, &Room::memberLeft, this, &UserListModel::userRemoved);
+        connect(m_currentRoom, &Room::memberNameAboutToUpdate, this, &UserListModel::userRemoved);
+        connect(m_currentRoom, &Room::memberNameUpdated, this, &UserListModel::userAdded);
         connect(m_currentRoom, &Room::memberListChanged, this, &UserListModel::membersChanged);
-        connect(m_currentRoom, &Room::memberAvatarChanged, this, &UserListModel::avatarChanged);
+        connect(m_currentRoom, &Room::memberAvatarUpdated, this, &UserListModel::avatarChanged);
         connect(m_currentRoom->connection(), &Connection::loggedOut, this,
                 [this] { setRoom(nullptr); });
-
-        filter({});
-        qCDebug(MODELS) << m_users.count() << "user(s) in the room";
+        doFilter({});
+        qCDebug(MODELS) << m_memberIds.count() << "member(s) in the room";
     }
     endResetModel();
 }
 
-Quotient::User* UserListModel::userAt(QModelIndex index)
+Quotient::RoomMember UserListModel::userAt(QModelIndex index) const
 {
-    if (index.row() < 0 || index.row() >= m_users.size())
-        return nullptr;
-    return m_users.at(index.row());
+    if (index.row() < 0 || index.row() >= m_memberIds.size())
+        return {};
+    return m_currentRoom->member(m_memberIds.at(index.row()));
 }
 
 QVariant UserListModel::data(const QModelIndex& index, int role) const
@@ -71,23 +69,22 @@ QVariant UserListModel::data(const QModelIndex& index, int role) const
     if( !index.isValid() )
         return QVariant();
 
-    if( index.row() >= m_users.count() )
+    if( index.row() >= m_memberIds.count() )
     {
         qCWarning(MODELS) << "UserListModel, something's wrong: index.row() >= "
                              "m_users.count()";
         return QVariant();
     }
-    auto user = m_users.at(index.row());
+    auto m = userAt(index);
     if( role == Qt::DisplayRole )
     {
-        return user->displayname(m_currentRoom);
+        return m.displayName();
     }
     const auto* view = static_cast<const QAbstractItemView*>(parent());
     if (role == Qt::DecorationRole) {
-        // Make user avatars 150% high compared to display names
+        // Convert avatar image to QIcon
         const auto dpi = view->devicePixelRatioF();
-        if (auto av = user->avatar(int(view->iconSize().height() * dpi),
-                                   m_currentRoom);
+        if (auto av = m.avatar(static_cast<int>(view->iconSize().height() * dpi), [] {});
             !av.isNull()) {
             av.setDevicePixelRatio(dpi);
             return QIcon(QPixmap::fromImage(av));
@@ -99,9 +96,8 @@ QVariant UserListModel::data(const QModelIndex& index, int role) const
 
     if (role == Qt::ToolTipRole)
     {
-        auto tooltip = QStringLiteral("<b>%1</b><br>%2")
-                .arg(user->name(m_currentRoom).toHtmlEscaped(),
-                     user->id().toHtmlEscaped());
+        auto tooltip =
+            QStringLiteral("<b>%1</b><br>%2").arg(m.name().toHtmlEscaped(), m.id().toHtmlEscaped());
         // TODO: Find a new way to determine that the user is bridged
 //        if (!user->bridged().isEmpty())
 //            tooltip += "<br>" + tr("Bridged from: %1").arg(user->bridged());
@@ -111,10 +107,10 @@ QVariant UserListModel::data(const QModelIndex& index, int role) const
     if (role == Qt::ForegroundRole) {
         // FIXME: boilerplate with TimelineItem.qml:57
         const auto& palette = view->palette();
-        return QColor::fromHslF(user->hueF(),
-            1 - palette.color(QPalette::Window).saturationF(),
-            0.9 - 0.7 * palette.color(QPalette::Window).lightnessF(),
-            palette.color(QPalette::ButtonText).alphaF());
+        return QColor::fromHslF(static_cast<float>(m.hueF()),
+                                1 - palette.color(QPalette::Window).saturationF(),
+                                0.9f - 0.7f * palette.color(QPalette::Window).lightnessF(),
+                                palette.color(QPalette::ButtonText).alphaF());
     }
 
     return QVariant();
@@ -125,38 +121,37 @@ int UserListModel::rowCount(const QModelIndex& parent) const
     if( parent.isValid() )
         return 0;
 
-    return m_users.count();
+    return m_memberIds.count();
 }
 
-void UserListModel::userAdded(Quotient::User* user)
+void UserListModel::userAdded(const RoomMember& member)
 {
-    auto pos = findUserPos(user);
-    if (pos != m_users.size() && m_users[pos] == user)
+    auto pos = findUserPos(member.id());
+    if (pos != m_memberIds.size() && m_memberIds[pos] == member.id())
     {
-        qCWarning(MODELS) << "Trying to add the user" << user->id()
+        qCWarning(MODELS) << "Trying to add the user" << member.id()
                           << "but it's already in the user list";
         return;
     }
     beginInsertRows(QModelIndex(), pos, pos);
-    m_users.insert(pos, user);
+    m_memberIds.insert(pos, member.id());
     endInsertRows();
 }
 
-void UserListModel::userRemoved(Quotient::User* user)
+void UserListModel::userRemoved(const RoomMember& member)
 {
-    auto pos = findUserPos(user);
-    if (pos == m_users.size())
+    auto pos = findUserPos(member);
+    if (pos == m_memberIds.size())
     {
         qCWarning(MODELS)
             << "Trying to remove a room member not in the user list:"
-            << user->id();
+            << member.id();
         return;
     }
 
     beginRemoveRows(QModelIndex(), pos, pos);
-    m_users.removeAt(pos);
+    m_memberIds.removeAt(pos);
     endRemoveRows();
-    user->disconnect(this);
 }
 
 void UserListModel::filter(const QString& filterString)
@@ -164,44 +159,51 @@ void UserListModel::filter(const QString& filterString)
     if (m_currentRoom == nullptr)
         return;
 
-    QElapsedTimer et; et.start();
-
     beginResetModel();
-    m_users.clear();
-    const auto all = m_currentRoom->users();
-    std::remove_copy_if(all.begin(), all.end(), std::back_inserter(m_users),
-        [&](User* u) {
-            return !(u->name(m_currentRoom).contains(filterString) ||
-                     u->id().contains(filterString));
-        });
-    std::sort(m_users.begin(), m_users.end(), m_currentRoom->memberSorter());
+    doFilter(filterString);
     endResetModel();
-
-    qCDebug(MODELS) << "Filtering" << m_users.size() << "user(s) in"
-                    << m_currentRoom->displayName() << "took" << et;
 }
 
-void UserListModel::refresh(Quotient::User* user, QVector<int> roles)
+void UserListModel::refresh(const RoomMember& member, QVector<int> roles)
 {
-    auto pos = findUserPos(user);
-    if ( pos != m_users.size() )
+    auto pos = findUserPos(member);
+    if ( pos != m_memberIds.size() )
         emit dataChanged(index(pos), index(pos), roles);
     else
         qCWarning(MODELS)
             << "Trying to access a room member not in the user list";
 }
 
-void UserListModel::avatarChanged(Quotient::User* user)
+void UserListModel::avatarChanged(const RoomMember& m)
 {
-    refresh(user, {Qt::DecorationRole});
+    refresh(m, {Qt::DecorationRole});
 }
 
-int UserListModel::findUserPos(User* user) const
+int UserListModel::findUserPos(const Quotient::RoomMember& m) const
 {
-    return findUserPos(m_currentRoom->disambiguatedMemberName(user->id()));
+    return findUserPos(m.disambiguatedName());
 }
 
 int UserListModel::findUserPos(const QString& username) const
 {
-    return m_currentRoom->memberSorter().lowerBoundIndex(m_users, username);
+    return static_cast<int>(Quotient::lowerBoundMemberIndex(m_memberIds, username, m_currentRoom));
+}
+
+void UserListModel::doFilter(const QString& filterString)
+{
+    QElapsedTimer et; et.start();
+
+    auto filteredMembers = Quotient::rangeTo<QList>(
+        std::views::filter(m_currentRoom->joinedMembers(),
+                           Quotient::memberMatcher(filterString, Qt::CaseInsensitive)));
+    std::ranges::sort(filteredMembers, Quotient::MemberSorter());
+    const auto sortedIds = std::views::transform(filteredMembers, &RoomMember::id);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+    m_memberIds.assign(sortedIds.begin(), sortedIds.end());
+#else
+    m_memberIds = QList(sortedIds.begin(), sortedIds.end());
+#endif
+
+    qCDebug(MODELS) << "Filtering" << m_memberIds.size() << "user(s) in"
+                    << m_currentRoom->displayName() << "took" << et;
 }
