@@ -16,40 +16,38 @@ namespace Quotient
 {
     class Connection;
     class Room;
-    class User;
+    class RoomMember;
 }
 
 class UserListModel: public QAbstractListModel
 {
         Q_OBJECT
     public:
-        using User = Quotient::User;
-
         UserListModel(QAbstractItemView* parent);
-        virtual ~UserListModel();
 
         void setRoom(Quotient::Room* room);
-        User* userAt(QModelIndex index);
+        Quotient::RoomMember userAt(QModelIndex index) const;
 
         QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
         int rowCount(const QModelIndex& parent=QModelIndex()) const override;
-    
+
     signals:
-        void membersChanged(); //< Reflection of Room::memberListChanged
+        void membersChanged(); //!< Reflection of Room::memberListChanged
 
     public slots:
         void filter(const QString& filterString);
 
     private slots:
-        void userAdded(User* user);
-        void userRemoved(User* user);
-        void refresh(User* user, QVector<int> roles = {});
-        void avatarChanged(User* user);
+        void userAdded(const Quotient::RoomMember& member);
+        void userRemoved(const Quotient::RoomMember& member);
+        void refresh(const Quotient::RoomMember& member, QVector<int> roles = {});
+        void avatarChanged(const Quotient::RoomMember& m);
 
     private:
         Quotient::Room* m_currentRoom;
-        QList<User*> m_users;
+        QList<QString> m_memberIds;
 
-        int findUserPos(User* user) const;
+        int findUserPos(const Quotient::RoomMember &m) const;
         int findUserPos(const QString& username) const;
+        void doFilter(const QString& filterString);
 };
