@@ -1,25 +1,17 @@
 /**************************************************************************
  *                                                                        *
- * Copyright (C) 2015 Felix Rohrbach <kde@fxrh.de>                        *
+ * SPDX-FileCopyrightText: 2015 Felix Rohrbach <kde@fxrh.de>              *
  *                                                                        *
- * This program is free software; you can redistribute it and/or          *
- * modify it under the terms of the GNU General Public License            *
- * as published by the Free Software Foundation; either version 3         *
- * of the License, or (at your option) any later version.                 *
- *                                                                        *
- * This program is distributed in the hope that it will be useful,        *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- * GNU General Public License for more details.                           *
- *                                                                        *
- * You should have received a copy of the GNU General Public License      *
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *                                                                        *
  **************************************************************************/
 
 #include "roomlistdock.h"
 
+#include "logging_categories.h"
+
 #include <QtWidgets/QMenu>
+#include <QtWidgets/QMessageBox>
 #include <QtWidgets/QStyledItemDelegate>
 #include <QtWidgets/QLabel>
 #include <QtWidgets/QPlainTextEdit>
@@ -31,8 +23,9 @@
 #include "models/orderbytag.h"
 #include "quaternionroom.h"
 #include "roomdialogs.h"
-#include <connection.h>
-#include <settings.h>
+
+#include <Quotient/connection.h>
+#include <Quotient/settings.h>
 
 using Quotient::SettingsGroup;
 
@@ -135,7 +128,8 @@ RoomListDock::RoomListDock(MainWindow* parent)
         const auto& i = model->index(groupPos, 0);
         const auto groupKey = model->roomGroupAt(i).toString();
         if (groupKey.startsWith("org.qmatrixclient"))
-            qCritical() << groupKey << "is deprecated!"; // Fighting the legacy
+            qCCritical(MAIN)
+                << groupKey << "is deprecated!"; // Fighting the legacy
         auto groupState = dockSettings.value(groupKey);
         if (!groupState.isValid()) {
             if (groupKey.startsWith(RoomGroup::SystemPrefix)) {
@@ -201,10 +195,14 @@ RoomListDock::RoomListDock(MainWindow* parent)
     forgetAction =
         roomContextMenu->addAction(QIcon::fromTheme("irc-remove-operator"),
         tr("Forget room"), this, [this] {
-            if (auto room = getSelectedRoom())
-            {
-                Q_ASSERT(room->connection());
-                room->connection()->forgetRoom(room->id());
+            if (auto room = getSelectedRoom()) {
+                QMessageBox::StandardButton confirmation = QMessageBox::question(
+                    this, tr("Forget this room?"),
+                    tr("Are you sure you want to forget room %1?").arg(room->displayName()));
+                if (confirmation == QMessageBox::Yes) {
+                    if (QUO_CHECK(room->connection()))
+                        room->connection()->forgetRoom(room->id());
+                }
             }
         });
 
@@ -224,6 +222,11 @@ void RoomListDock::addConnection(Quotient::Connection* connection)
     model->addConnection(connection);
 }
 
+void RoomListDock::deleteConnection(Quotient::Connection* connection)
+{
+    model->deleteConnection(connection);
+}
+
 void RoomListDock::updateSortingMode()
 {
 //    const auto sortMode =
@@ -319,13 +322,8 @@ void RoomListDock::addTagsSelected()
             return;
 
         auto tags = room->tags();
-        const auto enteredTags = tagsInput->toPlainText().split('\n',
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
-                                                                Qt::SkipEmptyParts
-#else
-                                                                QString::SkipEmptyParts
-#endif
-        );
+        const auto enteredTags =
+            tagsInput->toPlainText().split('\n', Qt::SkipEmptyParts);
         for (const auto& tag: enteredTags)
             tags[captionToTag(tag)]; // No overwriting, just ensure existence
 
