@@ -81,10 +81,6 @@
         <source>/%1 &lt;memberId&gt;</source>
         <translation>/%1 &lt;Benutzer-ID&gt;</translation>
     </message>
-    <message>
-        <source>Unknown /command. Use // to send this line literally</source>
-        <translation>Unbekannter /Befehl. Starte mit //, um diese Zeile normal zu senden</translation>
-    </message>
     <message>
         <source>Attach</source>
         <translation>Anhängen</translation>
@@ -145,8 +141,8 @@
         <translation>Aktuell tippen:</translation>
     </message>
     <message>
-        <source>At pos %1: %2</source>
-        <translation>An Position %1: %2</translation>
+        <source>At character %1: %2</source>
+        <translation>Bei Zeichen %1: %2</translation>
     </message>
     <message>
         <source>%L1 more</source>
@@ -160,92 +156,17 @@
         <source>Attaching the pasted image</source>
         <translation>Anhängen des eingefügten Bildes</translation>
     </message>
-</context>
-<context>
-    <name>Timeline</name>
     <message>
-        <source>(no topic)</source>
-        <translation>(kein Thema)</translation>
+        <source>Can&apos;t attach a file without a selected room</source>
+        <translation>Kann keine Datei ohne einen ausgewählten Raum anhängen</translation>
     </message>
     <message>
-        <source>Unknown</source>
-        <translation>Unbekannt</translation>
+        <source>Cannot insert HTML - it&apos;s either invalid or unsupported</source>
+        <translation>HTML kann nicht eingefügt werden – es ist entweder ungültig oder wird nicht unterstützt</translation>
     </message>
     <message>
-        <source>Unstable room version!</source>
-        <translation>Instabile Raumversion!</translation>
-    </message>
-    <message>
-        <source>(no name)</source>
-        <translation>(kein Name)</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln byte(s)</source>
-        <translation>
-            <numerusform>%Ln Byte</numerusform>
-            <numerusform>%Ln Bytes</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>%L1 MB</source>
-        <translation>%L1 MB</translation>
-    </message>
-    <message>
-        <source>%L1 GB</source>
-        <translation>%L1 GB</translation>
-    </message>
-    <message>
-        <source>This room has been upgraded.</source>
-        <translation>Der Raum wurde aktualisiert.</translation>
-    </message>
-    <message>
-        <source>Go to
-new room</source>
-        <translation>Gehe zu
-neuem Raum</translation>
-    </message>
-    <message>
-        <source>Room
-settings</source>
-        <translation>Raum-
-Einstellungen</translation>
-    </message>
-    <message>
-        <source>Latest events</source>
-        <translation>Neueste Ereignisse</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events back from now</source>
-        <translation>
-            <numerusform>%Ln Ereignis zurückgescrollt</numerusform>
-            <numerusform>%Ln Ereignisse zurückgescrollt</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>%L1 kB</source>
-        <translation>%L1 kB</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events cached</source>
-        <translation>
-            <numerusform>%Ln Ereignis zwischengespeichert</numerusform>
-            <numerusform>%Ln Ereignisse zwischengespeichert</numerusform>
-        </translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events requested from the server</source>
-        <translation>
-            <numerusform>%Ln Ereignis vom Server angefragt</numerusform>
-            <numerusform>%Ln Ereignisse vom Server angefragt</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>Hide topic</source>
-        <translation>Thema ausblenden</translation>
-    </message>
-    <message>
-        <source>Show topic</source>
-        <translation>Thema anzeigen</translation>
+        <source>Unknown /command. If you intended to send a message, start with // instead of /</source>
+        <translation>Unbekannter /Befehl. Um eine Nachricht zu senden, beginnen Sie mit // anstelle von /</translation>
     </message>
 </context>
 <context>
@@ -270,6 +191,10 @@ Einstellungen</translation>
         <source>Please fill the fields as desired. None are mandatory</source>
         <translation>Bitte füllen Sie die Felder wie gewünscht aus. Alle sind optional.</translation>
     </message>
+    <message>
+        <source>Remove</source>
+        <translation>Entfernen</translation>
+    </message>
 </context>
 <context>
     <name>Dialog</name>
@@ -368,10 +293,6 @@ Einstellungen</translation>
         <source>Saved device id</source>
         <translation>Gespeicherte Geräte-ID</translation>
     </message>
-    <message>
-        <source>Enable E2EE (BETA)</source>
-        <translation>E2EE einschalten (BETA)</translation>
-    </message>
 </context>
 <context>
     <name>MainWindow</name>
@@ -939,6 +860,13 @@ Matrix URI oder matrix.to Link</translation>
         <source>Show a confirmation box before opening non-Matrix links in an external application</source>
         <translation>Zeige ein Bestätigungsfeld vor dem Öffnen von Nicht-Matrix-Links in einer externen Anwendung an</translation>
     </message>
+    <message numerus="yes">
+        <source>Loading %Ln accounts, please wait</source>
+        <translation>
+            <numerusform>%Ln Konto wird geladen, bitte warten</numerusform>
+            <numerusform>%Ln Konten werden geladen, bitte warten</numerusform>
+        </translation>
+    </message>
     <message>
         <source>Account %1 is synchronised, have a good chat</source>
         <translation>Konto %1 ist synchronisiert, viel Spaß beim Chatten</translation>
@@ -962,6 +890,14 @@ Matrix URI oder matrix.to Link</translation>
         <source>Dock panels</source>
         <translation>Dock-Paneele</translation>
     </message>
+    <message>
+        <source>Can&apos;t find the event without knowing the room</source>
+        <translation>Kann das Ereignis nicht finden, ohne den Raum zu kennen</translation>
+    </message>
+    <message>
+        <source>Open the room that has this event to scroll to %1</source>
+        <translation>Öffnen Sie den Raum, in dem sich dieses Ereignis befindet, um zu %1 zu scrollen</translation>
+    </message>
 </context>
 <context>
     <name>MessageEventModel</name>
@@ -1132,6 +1068,10 @@ Matrix URI oder matrix.to Link</translation>
         <source>kicked %1 from the room</source>
         <translation>hat %1 aus dem Raum entfernt</translation>
     </message>
+    <message>
+        <source>(loading)</source>
+        <translation>(lädt)</translation>
+    </message>
 </context>
 <context>
     <name>NetworkConfigDialog</name>
@@ -1222,6 +1162,10 @@ Matrix URI oder matrix.to Link</translation>
         <source>You are using an UNSTABLE room version (%1). The server may stop supporting it at any moment. Do you still want to use this version?</source>
         <translation>Sie verwenden eine INSTABILE Raumversion (%1). Der Server kann die Unterstützung jederzeit einstellen. Möchten Sie diese Version noch verwenden?</translation>
     </message>
+    <message>
+        <source>(no available room versions)</source>
+        <translation>(keine verfügbaren Raumversionen)</translation>
+    </message>
 </context>
 <context>
     <name>RoomListDock</name>
@@ -1277,6 +1221,14 @@ Matrix URI oder matrix.to Link</translation>
         <source>Rooms (%L1)</source>
         <translation>Räume (%L1)</translation>
     </message>
+    <message>
+        <source>Forget this room?</source>
+        <translation>Diesen Raum vergessen?</translation>
+    </message>
+    <message>
+        <source>Are you sure you want to forget room %1?</source>
+        <translation>Sind Sie sicher, dass Sie den Raum %1 vergessen möchten?</translation>
+    </message>
 </context>
 <context>
     <name>RoomListModel</name>
@@ -1355,6 +1307,14 @@ Matrix URI oder matrix.to Link</translation>
         <source>Events after fully read marker: %L1</source>
         <translation>Ereignisse nach vollständig gelesenem Marker: %L1</translation>
     </message>
+    <message>
+        <source>Unread events/highlights since read receipt: %L1/%L2</source>
+        <translation>Ungelesene Ereignisse/Hervorhebungen nach vollständig gelesenem Marker: %L1/%L2</translation>
+    </message>
+    <message>
+        <source>Unread events since read receipt: %L1</source>
+        <translation>Ungelesene Ereignisse nach vollständig gelesenem Marker: %L1</translation>
+    </message>
     <message>
         <source>Room id: %1</source>
         <translation>Raum-ID: %1</translation>
@@ -1498,6 +1458,29 @@ Dieser Vorgang kann nicht rückgängig gemacht werden.</translation>
         <source>downloaded to %1</source>
         <translation>nach %1 heruntergeladen</translation>
     </message>
+    <message>
+        <source>Unknown</source>
+        <translation>Unbekannt</translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln byte(s)</source>
+        <translation>
+            <numerusform>%Ln Byte</numerusform>
+            <numerusform>%Ln Bytes</numerusform>
+        </translation>
+    </message>
+    <message>
+        <source>%L1 kB</source>
+        <translation>%L1 kB</translation>
+    </message>
+    <message>
+        <source>%L1 MB</source>
+        <translation>%L1 MB</translation>
+    </message>
+    <message>
+        <source>%L1 GB</source>
+        <translation>%L1 GB</translation>
+    </message>
 </context>
 <context>
     <name>TimelineItem</name>
@@ -1574,12 +1557,12 @@ neuem Raum</translation>
             <numerusform>%Ln Hervorhebungen</numerusform>
         </translation>
     </message>
-</context>
-<context>
-    <name>ThumbnailResponse</name>
-    <message>
-        <source>Image request has been cancelled</source>
-        <translation>Bildanfrage wurde abgebrochen</translation>
+    <message numerus="yes">
+        <source>%Ln unread message(s) across all rooms</source>
+        <translation>
+            <numerusform>%Ln ungelesene Nachricht in allen Räumen</numerusform>
+            <numerusform>%Ln ungelesene Nachrichten in allen Räumen</numerusform>
+        </translation>
     </message>
 </context>
 <context>
@@ -1639,10 +1622,6 @@ neuem Raum</translation>
 </context>
 <context>
     <name>ProfileDialog</name>
-    <message>
-        <source>This is the current device</source>
-        <translation>Dies ist das aktuelle Gerät</translation>
-    </message>
     <message>
         <source>Device display name</source>
         <translation>Anzeigename des Geräts</translation>
@@ -1679,10 +1658,6 @@ neuem Raum</translation>
         <source>Access token</source>
         <translation>Zugriffstoken</translation>
     </message>
-    <message>
-        <source>Apply and close</source>
-        <translation>Übernehmen und schließen</translation>
-    </message>
     <message>
         <source>Loading other devices...</source>
         <translation>Lade andere Geräte …</translation>
@@ -1695,6 +1670,54 @@ neuem Raum</translation>
         <source>Set avatar</source>
         <translation>Avatar einstellen</translation>
     </message>
+    <message>
+        <source>Cancel</source>
+        <translation>Abbrechen</translation>
+    </message>
+    <message>
+        <source>Please accept the verification request on the device you want to verify</source>
+        <translation>Bitte akzeptieren Sie die Verifizierungsanfrage auf dem Gerät, das Sie verifizieren möchten.</translation>
+    </message>
+    <message>
+        <source>This device</source>
+        <translation>Dieses Gerät</translation>
+    </message>
+    <message>
+        <source>No E2EE</source>
+        <translation>Keine E2EE</translation>
+    </message>
+    <message>
+        <source>Verification was cancelled on the other side</source>
+        <translation>Die Verifizierung wurde auf der Gegenseite abgebrochen</translation>
+    </message>
+</context>
+<context>
+    <name>Timeline</name>
+    <message>
+        <source>Latest events</source>
+        <translation>Neueste Ereignisse</translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events back from now</source>
+        <translation>
+            <numerusform>%Ln Ereignis zurückgescrollt</numerusform>
+            <numerusform>%Ln Ereignisse zurückgescrollt</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events cached</source>
+        <translation>
+            <numerusform>%Ln Ereignis zwischengespeichert</numerusform>
+            <numerusform>%Ln Ereignisse zwischengespeichert</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events requested from the server</source>
+        <translation>
+            <numerusform>%Ln Ereignis vom Server angefragt</numerusform>
+            <numerusform>%Ln Ereignisse vom Server angefragt</numerusform>
+        </translation>
+    </message>
 </context>
 <context>
     <name>ChatEdit</name>
@@ -1706,10 +1729,6 @@ neuem Raum</translation>
         <source>Reset the current character formatting to the default</source>
         <translation>Zurücksetzen der aktuellen Zeichenformatierung auf die Standardeinstellung</translation>
     </message>
-    <message>
-        <source>Could not insert HTML - it&apos;s either invalid or unsupported</source>
-        <translation>HTML konnte nicht eingefügt werden – es ist entweder ungültig oder wird nicht unterstützt</translation>
-    </message>
     <message>
         <source>Paste as rich text</source>
         <translation>Als Rich-Text einfügen</translation>
@@ -1747,14 +1766,49 @@ neuem Raum</translation>
     </message>
 </context>
 <context>
-    <name>AbstractThumbnailResponse</name>
+    <name>RoomHeader</name>
     <message>
-        <source>No connection to perform image request</source>
-        <translation>Keine Verbindung zum Durchführen einer Bildanforderung</translation>
+        <source>(no name)</source>
+        <translation>(kein Name)</translation>
     </message>
     <message>
-        <source>Image request is pending</source>
-        <translation>Bildanforderung ist ausstehend</translation>
+        <source>This room has been upgraded.</source>
+        <translation>Der Raum wurde aktualisiert.</translation>
+    </message>
+    <message>
+        <source>Unstable room version!</source>
+        <translation>Instabile Raumversion!</translation>
+    </message>
+    <message>
+        <source>(no topic)</source>
+        <translation>(kein Thema)</translation>
+    </message>
+    <message>
+        <source>Hide topic</source>
+        <translation>Thema ausblenden</translation>
+    </message>
+    <message>
+        <source>Show topic</source>
+        <translation>Thema anzeigen</translation>
+    </message>
+    <message>
+        <source>Room
+settings</source>
+        <translation>Raum-
+Einstellungen</translation>
+    </message>
+    <message>
+        <source>Go to
+new room</source>
+        <translation>Gehe zu
+neuem Raum</translation>
+    </message>
+</context>
+<context>
+    <name>VerificationDialog</name>
+    <message>
+        <source>Confirm that the same icons, in the same order are displayed on the other side</source>
+        <translation>Vergewissern Sie sich, dass die gleichen Symbole in der gleichen Reihenfolge auf der anderen Seite angezeigt werden</translation>
     </message>
 </context>
 </TS>
