@@ -81,10 +81,6 @@
         <source>/%1 &lt;memberId&gt;</source>
         <translation>/%1 &lt;memberId&gt;</translation>
     </message>
-    <message>
-        <source>Unknown /command. Use // to send this line literally</source>
-        <translation>Unknown /command. Use // to send this line literally.</translation>
-    </message>
     <message>
         <source>Attach</source>
         <translation>Attach</translation>
@@ -145,8 +141,8 @@
         <translation>Currently typing:</translation>
     </message>
     <message>
-        <source>At pos %1: %2</source>
-        <translation>At pos %1: %2</translation>
+        <source>At character %1: %2</source>
+        <translation>At character %1: %2</translation>
     </message>
     <message>
         <source>%L1 more</source>
@@ -160,92 +156,17 @@
         <source>Attaching the pasted image</source>
         <translation>Attaching the pasted image</translation>
     </message>
-</context>
-<context>
-    <name>Timeline</name>
     <message>
-        <source>(no topic)</source>
-        <translation>(no topic)</translation>
+        <source>Can&apos;t attach a file without a selected room</source>
+        <translation>Can&apos;t attach a file without a selected room</translation>
     </message>
     <message>
-        <source>Unknown</source>
-        <translation>Unknown</translation>
+        <source>Cannot insert HTML - it&apos;s either invalid or unsupported</source>
+        <translation>Cannot insert HTML — it&apos;s either invalid or unsupported</translation>
     </message>
     <message>
-        <source>Unstable room version!</source>
-        <translation>Unstable room version!</translation>
-    </message>
-    <message>
-        <source>(no name)</source>
-        <translation>(no name)</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln byte(s)</source>
-        <translation>
-            <numerusform>%Ln byte</numerusform>
-            <numerusform>%Ln bytes</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>%L1 MB</source>
-        <translation>%L1 MB</translation>
-    </message>
-    <message>
-        <source>%L1 GB</source>
-        <translation>%L1 GB</translation>
-    </message>
-    <message>
-        <source>This room has been upgraded.</source>
-        <translation>This room has been upgraded.</translation>
-    </message>
-    <message>
-        <source>Go to
-new room</source>
-        <translation>Go to
-new room</translation>
-    </message>
-    <message>
-        <source>Room
-settings</source>
-        <translation>Room
-settings</translation>
-    </message>
-    <message>
-        <source>Latest events</source>
-        <translation>Latest events</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events back from now</source>
-        <translation>
-            <numerusform>%Ln events back from now</numerusform>
-            <numerusform>%Ln events back from now</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>%L1 kB</source>
-        <translation>%L1 kB</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events cached</source>
-        <translation>
-            <numerusform>%Ln event cached</numerusform>
-            <numerusform>%Ln events cached</numerusform>
-        </translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events requested from the server</source>
-        <translation>
-            <numerusform>%Ln event requested from the server</numerusform>
-            <numerusform>%Ln events requested from the server</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>Hide topic</source>
-        <translation>Hide topic</translation>
-    </message>
-    <message>
-        <source>Show topic</source>
-        <translation>Show topic</translation>
+        <source>Unknown /command. If you intended to send a message, start with // instead of /</source>
+        <translation>Unknown /command. If you intended to send a message, start with // instead of /</translation>
     </message>
 </context>
 <context>
@@ -270,6 +191,10 @@ settings</translation>
         <source>Please fill the fields as desired. None are mandatory</source>
         <translation>Please fill the fields as desired. None are mandatory</translation>
     </message>
+    <message>
+        <source>Remove</source>
+        <translation>Remove</translation>
+    </message>
 </context>
 <context>
     <name>Dialog</name>
@@ -368,10 +293,6 @@ settings</translation>
         <source>Saved device id</source>
         <translation>Saved device id</translation>
     </message>
-    <message>
-        <source>Enable E2EE (BETA)</source>
-        <translation>Enable E2EE (BETA)</translation>
-    </message>
 </context>
 <context>
     <name>MainWindow</name>
@@ -969,6 +890,14 @@ Matrix URI or matrix.to link</translation>
         <source>Dock panels</source>
         <translation>Dock panels</translation>
     </message>
+    <message>
+        <source>Can&apos;t find the event without knowing the room</source>
+        <translation>Can&apos;t find the event without knowing the room</translation>
+    </message>
+    <message>
+        <source>Open the room that has this event to scroll to %1</source>
+        <translation>Open the room that has this event to scroll to %1</translation>
+    </message>
 </context>
 <context>
     <name>MessageEventModel</name>
@@ -1139,6 +1068,14 @@ Matrix URI or matrix.to link</translation>
         <source>kicked %1 from the room</source>
         <translation>kicked %1 from the room</translation>
     </message>
+    <message>
+        <source>(loading)</source>
+        <translation>(loading)</translation>
+    </message>
+    <message>
+        <source>Could not decrypt the event</source>
+        <translation>Could not decrypt the event</translation>
+    </message>
 </context>
 <context>
     <name>NetworkConfigDialog</name>
@@ -1229,6 +1166,10 @@ Matrix URI or matrix.to link</translation>
         <source>You are using an UNSTABLE room version (%1). The server may stop supporting it at any moment. Do you still want to use this version?</source>
         <translation>You are using an UNSTABLE room version (%1). The server may stop supporting it at any moment. Do you still want to use this version?</translation>
     </message>
+    <message>
+        <source>(no available room versions)</source>
+        <translation>(no available room versions)</translation>
+    </message>
 </context>
 <context>
     <name>RoomListDock</name>
@@ -1284,6 +1225,14 @@ Matrix URI or matrix.to link</translation>
         <source>Rooms (%L1)</source>
         <translation>Rooms (%L1)</translation>
     </message>
+    <message>
+        <source>Forget this room?</source>
+        <translation>Forget this room?</translation>
+    </message>
+    <message>
+        <source>Are you sure you want to forget room %1?</source>
+        <translation>Are you sure you want to forget room %1?</translation>
+    </message>
 </context>
 <context>
     <name>RoomListModel</name>
@@ -1513,6 +1462,29 @@ This operation cannot be reverted.</translation>
         <source>downloaded to %1</source>
         <translation>downloaded to %1</translation>
     </message>
+    <message>
+        <source>Unknown</source>
+        <translation>Unknown</translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln byte(s)</source>
+        <translation>
+            <numerusform>%Ln byte</numerusform>
+            <numerusform>%Ln bytes</numerusform>
+        </translation>
+    </message>
+    <message>
+        <source>%L1 kB</source>
+        <translation>%L1 kB</translation>
+    </message>
+    <message>
+        <source>%L1 MB</source>
+        <translation>%L1 MB</translation>
+    </message>
+    <message>
+        <source>%L1 GB</source>
+        <translation>%L1 GB</translation>
+    </message>
 </context>
 <context>
     <name>TimelineItem</name>
@@ -1589,12 +1561,12 @@ new room</translation>
             <numerusform>%Ln highlights</numerusform>
         </translation>
     </message>
-</context>
-<context>
-    <name>ThumbnailResponse</name>
-    <message>
-        <source>Image request has been cancelled</source>
-        <translation>Image request has been cancelled</translation>
+    <message numerus="yes">
+        <source>%Ln unread message(s) across all rooms</source>
+        <translation>
+            <numerusform>%Ln unread messages across all rooms</numerusform>
+            <numerusform>%Ln unread messages across all rooms</numerusform>
+        </translation>
     </message>
 </context>
 <context>
@@ -1654,10 +1626,6 @@ new room</translation>
 </context>
 <context>
     <name>ProfileDialog</name>
-    <message>
-        <source>This is the current device</source>
-        <translation>This is the current device</translation>
-    </message>
     <message>
         <source>Device display name</source>
         <translation>Device display name</translation>
@@ -1694,10 +1662,6 @@ new room</translation>
         <source>Access token</source>
         <translation>Access token</translation>
     </message>
-    <message>
-        <source>Apply and close</source>
-        <translation>Apply and close</translation>
-    </message>
     <message>
         <source>Loading other devices...</source>
         <translation>Loading other devices...</translation>
@@ -1710,6 +1674,78 @@ new room</translation>
         <source>Set avatar</source>
         <translation>Set avatar</translation>
     </message>
+    <message>
+        <source>Verification timed out</source>
+        <translation>Verification timed out</translation>
+    </message>
+    <message>
+        <source>Verification was cancelled</source>
+        <translation>Verification was cancelled</translation>
+    </message>
+    <message>
+        <source>Verification did not succeed</source>
+        <translation>Verification did not succeed</translation>
+    </message>
+    <message>
+        <source>Cancel</source>
+        <translation>Cancel</translation>
+    </message>
+    <message>
+        <source>Please accept the verification request on the device you want to verify</source>
+        <translation>Please accept the verification request on the device you want to verify</translation>
+    </message>
+    <message>
+        <source>This device</source>
+        <translation>This device</translation>
+    </message>
+    <message>
+        <source>Verified</source>
+        <translation>Verified</translation>
+    </message>
+    <message>
+        <source>Verify...</source>
+        <translation>Verify...</translation>
+    </message>
+    <message>
+        <source>No E2EE</source>
+        <translation>No E2EE</translation>
+    </message>
+    <message>
+        <source>Verification failed: icons did not match</source>
+        <translation>Verification failed: icons did not match</translation>
+    </message>
+    <message>
+        <source>Verification was cancelled on the other side</source>
+        <translation>Verification was cancelled on the other side</translation>
+    </message>
+</context>
+<context>
+    <name>Timeline</name>
+    <message>
+        <source>Latest events</source>
+        <translation>Latest events</translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events back from now</source>
+        <translation>
+            <numerusform>%Ln events back from now</numerusform>
+            <numerusform>%Ln events back from now</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events cached</source>
+        <translation>
+            <numerusform>%Ln event cached</numerusform>
+            <numerusform>%Ln events cached</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events requested from the server</source>
+        <translation>
+            <numerusform>%Ln event requested from the server</numerusform>
+            <numerusform>%Ln events requested from the server</numerusform>
+        </translation>
+    </message>
 </context>
 <context>
     <name>ChatEdit</name>
@@ -1721,10 +1757,6 @@ new room</translation>
         <source>Reset the current character formatting to the default</source>
         <translation>Reset the current character formatting to the default</translation>
     </message>
-    <message>
-        <source>Could not insert HTML - it&apos;s either invalid or unsupported</source>
-        <translation>Could not insert HTML - it&apos;s either invalid or unsupported</translation>
-    </message>
     <message>
         <source>Paste as rich text</source>
         <translation>Paste as rich text</translation>
@@ -1762,14 +1794,61 @@ new room</translation>
     </message>
 </context>
 <context>
-    <name>AbstractThumbnailResponse</name>
+    <name>RoomHeader</name>
     <message>
-        <source>No connection to perform image request</source>
-        <translation>No connection to perform image request</translation>
+        <source>(no name)</source>
+        <translation>(no name)</translation>
     </message>
     <message>
-        <source>Image request is pending</source>
-        <translation>Image request is pending</translation>
+        <source>This room has been upgraded.</source>
+        <translation>This room has been upgraded.</translation>
+    </message>
+    <message>
+        <source>Unstable room version!</source>
+        <translation>Unstable room version!</translation>
+    </message>
+    <message>
+        <source>(no topic)</source>
+        <translation>(no topic)</translation>
+    </message>
+    <message>
+        <source>Hide topic</source>
+        <translation>Hide topic</translation>
+    </message>
+    <message>
+        <source>Show topic</source>
+        <translation>Show topic</translation>
+    </message>
+    <message>
+        <source>Room
+settings</source>
+        <translation>Room
+settings</translation>
+    </message>
+    <message>
+        <source>Go to
+new room</source>
+        <translation>Go to
+new room</translation>
+    </message>
+</context>
+<context>
+    <name>VerificationDialog</name>
+    <message>
+        <source>Verifying device %1</source>
+        <translation>Verifying device %1</translation>
+    </message>
+    <message>
+        <source>They match</source>
+        <translation>They match</translation>
+    </message>
+    <message>
+        <source>They DON&apos;T match</source>
+        <translation>They DON&apos;T match</translation>
+    </message>
+    <message>
+        <source>Confirm that the same icons, in the same order are displayed on the other side</source>
+        <translation>Confirm that the same icons, in the same order are displayed on the other side</translation>
     </message>
 </context>
 </TS>
