@@ -81,10 +81,6 @@
         <source>/%1 &lt;memberId&gt;</source>
         <translation>/%1 &lt;memberId&gt;</translation>
     </message>
-    <message>
-        <source>Unknown /command. Use // to send this line literally</source>
-        <translation>/command desconocido. Utilice // para enviar esta línea literalmente</translation>
-    </message>
     <message>
         <source>Attach</source>
         <translation>Adjuntar</translation>
@@ -134,68 +130,6 @@
         <translation>Actualmente escribiendo:</translation>
     </message>
 </context>
-<context>
-    <name>Timeline</name>
-    <message>
-        <source>(no topic)</source>
-        <translation>(sin tema)</translation>
-    </message>
-    <message>
-        <source>Unknown</source>
-        <translation>Desconocido</translation>
-    </message>
-    <message>
-        <source>Unstable room version!</source>
-        <translation>¡Versión de sala inestable!</translation>
-    </message>
-    <message>
-        <source>(no name)</source>
-        <translation>(sin nombre)</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln byte(s)</source>
-        <translation>
-            <numerusform>%Ln byte</numerusform>
-            <numerusform>%Ln bytes</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>%L1 MB</source>
-        <translation>%L1 MB</translation>
-    </message>
-    <message>
-        <source>%L1 GB</source>
-        <translation>%L1 GB</translation>
-    </message>
-    <message>
-        <source>This room has been upgraded.</source>
-        <translation>Esta sala ha sido mejorada.</translation>
-    </message>
-    <message>
-        <source>Room
-settings</source>
-        <translation>Configuración
-de sala</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events back from now</source>
-        <translation>
-            <numerusform>%Ln evento desde ahora</numerusform>
-            <numerusform>%Ln eventos desde ahora</numerusform>
-        </translation>
-    </message>
-    <message>
-        <source>%L1 kB</source>
-        <translation>%L1 kB</translation>
-    </message>
-    <message numerus="yes">
-        <source>%Ln events cached</source>
-        <translation>
-            <numerusform>%Ln evento en caché</numerusform>
-            <numerusform>%Ln eventos en caché</numerusform>
-        </translation>
-    </message>
-</context>
 <context>
     <name>CreateRoomDialog</name>
     <message>
@@ -264,6 +198,10 @@ de sala</translation>
         <source>Restoring access, please wait</source>
         <translation>Restaurando el acceso, por favor espere</translation>
     </message>
+    <message>
+        <source>(none)</source>
+        <translation>(ninguno)</translation>
+    </message>
 </context>
 <context>
     <name>MainWindow</name>
@@ -687,6 +625,10 @@ Matrix URI o enlace matriz.to</translation>
         <source>Alexey &quot;Kitsune&quot; Rusakov</source>
         <translation>Alexey &quot;Kitsune&quot; Rusakov</translation>
     </message>
+    <message>
+        <source>&amp;Member list</source>
+        <translation>&amp;Lista de miembros</translation>
+    </message>
 </context>
 <context>
     <name>MessageEventModel</name>
@@ -1180,6 +1122,22 @@ Esta operación no se puede revertir.</translation>
         <source>downloaded to %1</source>
         <translation>descargado a %1</translation>
     </message>
+    <message>
+        <source>Unknown</source>
+        <translation>Desconocido</translation>
+    </message>
+    <message>
+        <source>%L1 kB</source>
+        <translation>%L1 kB</translation>
+    </message>
+    <message>
+        <source>%L1 MB</source>
+        <translation>%L1 MB</translation>
+    </message>
+    <message>
+        <source>%L1 GB</source>
+        <translation>%L1 GB</translation>
+    </message>
 </context>
 <context>
     <name>TimelineItem</name>
@@ -1225,26 +1183,12 @@ Esta operación no se puede revertir.</translation>
         </translation>
     </message>
 </context>
-<context>
-    <name>ThumbnailResponse</name>
-    <message>
-        <source>Image request has been cancelled</source>
-        <translation>La solicitud de imagen ha sido cancelada</translation>
-    </message>
-</context>
-<context>
-    <name>ProfileDialog</name>
-    <message>
-        <source>Account</source>
-        <translation>Cuenta</translation>
-    </message>
-    <message>
-        <source>Access token</source>
-        <translation>Token de acceso</translation>
-    </message>
-</context>
 <context>
     <name>TimelineWidget</name>
+    <message>
+        <source>Save file as...</source>
+        <translation>Guardar archivo como...</translation>
+    </message>
     <message>
         <source>Save file as</source>
         <translation>Guardar archivo como</translation>
@@ -1262,4 +1206,57 @@ Esta operación no se puede revertir.</translation>
         <translation>Abrir externamente</translation>
     </message>
 </context>
+<context>
+    <name>ProfileDialog</name>
+    <message>
+        <source>Account</source>
+        <translation>Cuenta</translation>
+    </message>
+    <message>
+        <source>Access token</source>
+        <translation>Token de acceso</translation>
+    </message>
+</context>
+<context>
+    <name>Timeline</name>
+    <message numerus="yes">
+        <source>%Ln events back from now</source>
+        <translation>
+            <numerusform>%Ln evento desde ahora</numerusform>
+            <numerusform>%Ln eventos desde ahora</numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>%Ln events cached</source>
+        <translation>
+            <numerusform>%Ln evento en caché</numerusform>
+            <numerusform>%Ln eventos en caché</numerusform>
+        </translation>
+    </message>
+</context>
+<context>
+    <name>RoomHeader</name>
+    <message>
+        <source>(no name)</source>
+        <translation>(sin nombre)</translation>
+    </message>
+    <message>
+        <source>This room has been upgraded.</source>
+        <translation>Esta sala ha sido mejorada.</translation>
+    </message>
+    <message>
+        <source>Unstable room version!</source>
+        <translation>¡Versión de sala inestable!</translation>
+    </message>
+    <message>
+        <source>(no topic)</source>
+        <translation>(sin tema)</translation>
+    </message>
+    <message>
+        <source>Room
+settings</source>
+        <translation>Configuración
+de sala</translation>
+    </message>
+</context>
 </TS>
